@@ -19,10 +19,16 @@ sub new {
         $1 . 'xmlns=""';
     }e;
     
+    if ($self->{engine}{parser}->get_option('recover')) {
+        $source =~ s/&(?!\w+;|#(?:x[a-fA-F0-9]+|\d+);)/&amp;/g;
+    }
+
     $self->{dom} = $self->{engine}{parser}->parse_string($source);
     $self;
 }
 
+sub dom { $_[0]->{dom} }
+
 sub process {
     my ($self, $vars) = @_;
     $self->_query($self->{dom}, $vars);
@@ -30,46 +36,83 @@ sub process {
 }
 
 sub as_string {
-    my ($self) = @_;
+    my ($self, %opt) = @_;
+    
+    $opt{is_xhtml} = 1 unless defined $opt{is_xhtml};
+
     if ($self->{source} =~ /^<\?xml/) {
         return $self->{dom}->serialize(1);
     }
     else { # for skip <?xml declaration.
-           # I know html_parse_string->toString, but it doesn't do I want.
         my $r = "";
         if (my $dtd = $self->{dom}->internalSubset) {
             $r = $dtd->serialize . "\n";
+        } elsif($opt{is_xhtml}) {
+            $self->{dom}->createInternalSubset(
+                'html',
+                '-//W3C//DTD XHTML 1.0 Transitional//EN',
+                'http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd',
+            );
+            $self->{dom_hacked}++;
         }
+
         if (my $root = $self->{dom}->documentElement) {
-            $r .= $root->serialize(1);
+            $r .= $root->serialize;
             $r =~ s/\n*$/\n/;
             
             if ($self->{xmlns_hacked}) {
                 $r =~ s{(<html[^>]+?)xmlns=""}{$1xmlns="http://www.w3.org/1999/xhtml"};
             }
         }
+        if ($self->{dom_hacked}) {
+            $self->{dom}->removeInternalSubset;
+            $self->{dom_hacked} = 0;
+        }
+        
         return $r;
     }
 }
 
+my $element_with_attr_regex = qr{
+    ^
+    \s*
+    (
+        \@[^@]+? |
+        (?:
+            (?: [^"]+? | .+?"[^"]+".+? )
+            (?: \@[^@]+? )?
+        )
+    )
+    \s*
+    (?: , | $ )
+}x;
+
 sub _exp_to_xpath {
     my ($self, $exp) = @_;
     return unless $exp;
     
     my $xpath;
-    if ($exp =~ m{^/}) {
+    if ($exp =~ m{^(?:/|\.(?:/|$))}) {
         $xpath = $exp;
     } elsif ($exp =~ m{^id\(}) {
         $xpath = $exp;
         $xpath =~ s{^id\((.+?)\)}{//\*\[\@id=$1\]}g; # id() hack
-    } else { # css selector
-        my ($elem, $attr) = $exp =~ m{(.*?)/?(@[^/]+)?$}; # extends @attr syntax
-        if ($elem) {
-            $xpath = HTML::Selector::XPath::selector_to_xpath($elem);
-            $xpath .= "/$attr" if $attr;
-        } elsif ($attr) {
-            $xpath = "//$attr";
+    } else {
+        # css selector extends @attr syntax
+        my @x;
+        while ($exp =~ s/$element_with_attr_regex//) {
+            my $e = $1;
+            my ($elem, $attr) = $e =~ m{(.*?)/?(@[^/@]+)?$};
+            my $x;
+            if ($elem) {
+                my $x = HTML::Selector::XPath::selector_to_xpath($elem);
+                   $x .= "/$attr" if $attr;
+                push @x, $x;
+            } elsif ($attr) {
+                push @x, "//$attr";
+            }
         }
+        $xpath = join " | ", @x;
     }
     $xpath;
 }
@@ -100,15 +143,9 @@ sub _assign_value {
             if (not $node->isa('XML::LibXML::Element')) {
                 croak "Can't assign hashref to " . ref($node);
             }
-            
-            my $fixed_value = { };
-            my $prefix_xpath = '/' . $node->nodeName;
-            for my $exp (keys %$value) {
-                $fixed_value->{ $prefix_xpath . $self->_exp_to_xpath($exp) } = delete $value->{$exp};
-            }
              
             my $parted = $self->_to_node($node->serialize);
-            $self->_query($parted, $fixed_value);
+            $self->_query($parted, $value);
             $node->replaceNode($parted);
         }
     }
@@ -116,7 +153,7 @@ sub _assign_value {
     elsif ($value_type eq 'ARRAY' and ref($value->[0]) eq 'HASH') { # => sub query loop
         for my $node (@$nodes) {
             if (not $node->isa('XML::LibXML::Element')) {
-                croak "Can't assign arrayref of hashrefs to " . ref($node);
+                croak "Can't assign loop list to " . ref($node);
             }
             
             my $container = XML::LibXML::DocumentFragment->new;
@@ -175,22 +212,25 @@ sub _assign_value {
     elsif (blessed($value) and $value->isa('Template::Semantic::Document')) { # => insert result
         for my $node (@$nodes) {
             if (not $node->isa('XML::LibXML::Element')) {
-                croak "Can't assign Template::Semantic::Document object to " . ref($node);
+                croak "Can't assign Template::Semantic::Document to " . ref($node);
             }
             $self->_replace_node($node, $value->{dom}->childNodes);
         }
     }
+    elsif (blessed($value) and $value->isa('XML::LibXML::Node')) { # => as LibXML object
+        if ($value->isa('XML::LibXML::Attr')) {
+            croak "Can't assign XML::LibXML::Attr to any element";
+        }
+        for my $node (@$nodes) {
+            $self->_replace_node($node, $value);
+        }
+    }
     elsif ($value_type eq 'SCALAR') { # => as HTML/XML
         my $root = $self->_to_node("<root>${$value}</root>");
         for my $node (@$nodes) {
             $self->_replace_node($node, $root->childNodes);
         }
     }
-    elsif (blessed($value) and $value->isa('XML::LibXML::Node')) { # => as LibXML object
-        for my $node (@$nodes) {
-            $self->_replace_node($node, $value);
-        }
-    }
     else { # => text or unknown(stringify)
         my $value = XML::LibXML::Text->new("$value");
         for my $node (@$nodes) {
@@ -199,10 +239,19 @@ sub _assign_value {
     }
 }
 
+sub _to_node {
+     my ($self, $xmlpart) = @_;
+     $self->{engine}{parser}->parse_string($xmlpart)->documentElement;
+}
+
 sub _replace_node {
     my ($self, $node, @replace) = @_;
     
-    if ($node->isa('XML::LibXML::Attr')) {
+    if ($node->isa('XML::LibXML::Element')) {
+        $node->removeChildNodes;
+        $node->addChild($_->cloneNode(1)) for @replace;
+    }
+    elsif ($node->isa('XML::LibXML::Attr')) {
         $node->setValue(join "", map { $_->textContent } @replace);
     }
     elsif ($node->isa('XML::LibXML::Comment')
@@ -212,15 +261,6 @@ sub _replace_node {
     elsif ($node->isa('XML::LibXML::Text')) {
         $node->setData(join "",  map { $_->textContent } @replace);
     }
-    else {
-        $node->removeChildNodes;
-        $node->addChild($_->cloneNode(1)) for @replace;
-    }
-}
-
-sub _to_node {
-     my ($self, $xmlpart) = @_;
-     $self->{engine}{parser}->parse_string($xmlpart)->documentElement;
 }
 
 sub _serialize_inner {
@@ -228,12 +268,15 @@ sub _serialize_inner {
     my $inner = "";
     if ($node->isa('XML::LibXML::Attr')) {
         $inner = $node->value;
-    } elsif ($node->isa('XML::LibXML::Comment')
-          or $node->isa('XML::LibXML::CDATASection')) {
+    }
+    elsif ($node->isa('XML::LibXML::Comment')
+        or $node->isa('XML::LibXML::CDATASection')) {
         $inner = $node->data;
-    } elsif ($node->isa('XML::LibXML::Text')) {
+    }
+    elsif ($node->isa('XML::LibXML::Text')) {
         $inner = $node->serialize;
-    } else {
+    }
+    else {
         $inner .= $_->serialize for $node->childNodes;
     }
     $inner;
@@ -252,9 +295,6 @@ Template::Semantic::Document - Template::Semantic Result object
       'title, h1' => 'foo',
   });
   
-  print $out;
-  print $out->as_string; # same as avobe
-  
   my $out = Template::Semantic->process('template.html', {
       '.foo, .bar' => 'baz',
       '.mee@class' => 'moo',
@@ -265,23 +305,68 @@ Template::Semantic::Document - Template::Semantic Result object
       '.foo' => sub { uc },
       '.bar' => sub { lc },
   });
+  
+  print $out;
+  print $out->as_string; # same as avobe
 
 =head1 METHODS
 
 =over 4
 
-=item $out->as_string
+=item $out = $out->process( \%vars )
 
-Returns the result as XHTML/XML.
+Process again to the result and returns L<Template::Semantic::Document>
+object again. So you can chain C<< ->process(...)->process(...) >>.
 
 =item "$out" (stringify)
 
 Calls C<as_string()> internally.
 
-=item $out = $out->process( \%vars )
+=item $html = $out->as_string( %options )
 
-Process again to the result and returns L<Template::Semantic::Document> object
-again. So you can chain like C<< ->process(...)->process(...) >>.
+Returns the result as XHTML/XML.
+
+=over 4
+
+=item * is_xhtml => [1|0]
+
+Default value is true. Even if DTD is not defined in the template, outputs as XHTML.
+When sets C<is_xhtml> false, skip this effect.
+
+  my $out = $ts->process(\<<END);
+  <div>
+      <img src="foo" />
+      <br />
+      <textarea></textarea>
+  </div>
+  END
+  ;
+  
+  print $out;
+  
+  <div>
+      <img src="foo" />
+      <br />
+      <textarea></textarea>
+  </div>
+  
+  print $out->as_string(is_xhtml => 0);
+  
+  <div>
+      <img src="foo"/>
+      <br/>
+      <textarea/>
+  </div>
+
+=back
+
+=item $dom = $out->dom()
+
+  my $out  = Template::Semantic->process($template, ...);
+  my $dom  = $out->dom;
+  my $root = $dom->documentElement; # get root element
+
+Gets the result as L<XML::LibXML::Document>.
 
 =back
 
